
local M = {}

-- NOTE: In this implementation, stack is an "upside down" FILO (First In, Last Out) stack
--       Anything PUSHed to the stack, goes to the end
--       Anything POPed from the stack is taken from the end
--       That way, FILO is achieved without needing to reindex the stack constantly
stack = {}


function M.draw()
    for i = 1, #stack do
        stack[i].draw() -- the stack is draw oldest to newest (in this case index 1 to #stack)
    end
end

function M.pop()
    stack[#stack] = nil
end

function M.popAll()
    stack = {}
end

function M.push(layer)
    stack[#stack + 1] = layer
end

function M.keypressed(key)
    for i = #stack, 1, -1 do
        if not stack[i].keypressed(key) then return end -- consume the keypress
    end
    state.current.keypressed(key)
end

function M.update(dt)
    for i = 1, #stack do
        stack[i].update(dt)
    end
    state.current.update(dt)
end

return M
